#!/usr/bin/env python
"""Unit tests for aho-corasick keyword string searching.

Danny Yoo (dyoo@hkn.eecs.berkeley.edu)
"""

import unittest



import ahocorasick


class AhoCorasickTest(unittest.TestCase):
    def setUp(self):
        self.tree = ahocorasick.KeywordTree()

    def tearDown(self):
        self.tree = None


    def testAnotherSearch(self):
        """Just to triangulate the search code.  We want to make sure
        that the implementation can do more than one search, at
        least."""
        self.tree.add("Python")
        self.tree.add("PLT Scheme")
        self.tree.make()
        self.assertEqual((19, 25),
                         self.tree.search("I am learning both Python and PLT Scheme"))
        self.assertEqual((0, 10),
                         self.tree.search("PLT Scheme is an interesting language."))


    def testSimpleConstruction(self):
        self.tree.add("foo")
        self.tree.add("bar")
        self.tree.make()
        self.assertEqual((10, 13),
                         self.tree.search("this is a foo message"))


    def testLongestSearch(self):
        self.tree.add("a")
        self.tree.add("alphabet");
        self.tree.make()
        self.assertEqual((0, 1), self.tree.search("alphabet soup"));
        self.assertEqual((0, 8), self.tree.search_long("alphabet soup"))
        self.assertEqual((13, 14), self.tree.search_long("yummy, I see an alphabet soup bowl"))
        


    def testSearchWithWholeMatch(self):
        """Make sure that longest search will match the whole string."""
        longString = "supercalifragilisticexpialidocious"
        self.tree.add(longString)
        self.tree.make()
        self.assertEqual((0, len(longString)),
                         self.tree.search(longString))


    def testSearchLongWithWholeMatch(self):
        """Make sure that longest search will match the whole string."""
        longString = "supercalifragilisticexpialidocious"
        self.tree.add(longString)
        self.tree.make()
        self.assertEqual((0, len(longString)),
                         self.tree.search_long(longString))

    def testSearchLongWithNoMatch(self):
        self.tree.add("foobar")
        self.tree.make()
        self.assertEqual(None, self.tree.search_long("fooba"))


    def testWithExpectedNonmatch(self):
        """Check to see that we don't always get a successful match."""
        self.tree.add("wise man")
        self.tree.make()
        self.assertEqual(
            None,
            self.tree.search("where fools and wise men fear to tread"))        


    def testEmptyConstruction(self):
        """Make sure that we can safely construct and dealloc a tree
        with no initial keywords.  Important because the C
        implementation assumes keywords exist on its dealloc, so we
        have to do some work on the back end to avoid silly segmentation
        errors."""
        tree = ahocorasick.KeywordTree()
        del tree


    def testEmptyMake(self):
        """Calling make() without adding any keywords should raise an
        AssertionError."""
        self.assertRaises(AssertionError, self.tree.make)


    def testDegenerateConstruction(self):
        """If we try searching before making, we should get an
        assertion error."""
        self.assertRaises(AssertionError,
                          lambda: self.tree.search("this is a test"))

if __name__ == '__main__':
    unittest.main()
